@@ -12,34 +12,11 @@ import scipy.stats as ss
 import matplotlib.pyplot as plt
 from lhd import lhd
 
-__version_info__ = (0, 9, 7)
+__version_info__ = (0, 10)
 __version__ = '.'.join(map(str, __version_info__))
 
 __author__ = 'Abraham Lee'
 
-#__all__ = [
-    ## core functions
-    #'uv', 'covariance_matrix', 'correlation_matrix',
-    ## continuous distribution constructors
-    #'N',
-    #'U',
-    #'Exp',
-    #'Gamma',
-    #'Beta',
-    #'LogN',
-    #'Chi2',
-    #'F',
-    #'Tri',
-    #'T',
-    #'Weib',
-    ## discrete distribution constructors
-    #'Bern',
-    #'B',
-    #'G',
-    #'H',
-    #'Pois'
-    #]
-
 npts = 10000
 
 CONSTANT_TYPES = (float, int, long)
@@ -158,7 +135,41 @@ class UncertainFunction(object):
         sk = self.skew
         kt = self.kurt
         return [mn, vr, sk, kt]
+    
+    def percentile(self, val):
+        """
+        Get the distribution value at a given percentile or set of percentiles.
+        This follows the NIST method for calculating percentiles.
         
+        Parameters
+        ----------
+        val : scalar or array
+            Either a single value or an array of values between 0 and 1.
+        
+        Returns
+        -------
+        out : scalar or array
+            The actual distribution value that appears at the requested
+            percentile value or values
+            
+        """
+        try:
+            # test to see if an input is given as an array
+            out = [self.percentile(vi) for vi in val]
+        except (ValueError, TypeError):
+            if val<=0:
+                out = float(min(self._mcpts))
+            elif val>=1:
+                out = float(max(self._mcpts))
+            else:
+                tmp = np.sort(self._mcpts)
+                n = val*(len(tmp) + 1)
+                k, d = int(n), n - int(n)
+                out = float(tmp[k] + d*(tmp[k + 1] - tmp[k]))
+        if isinstance(val, np.ndarray):
+            out = np.array(out)
+        return out
+    
     def _to_general_representation(self,str_func):
         mn, vr, sk, kt = self.stats
         return ('uv({:}, {:}, {:}, {:})'.format(
@@ -245,21 +256,21 @@ class UncertainFunction(object):
         if hist:
             h = plt.hist(vals, bins=np.round(np.sqrt(len(vals))), 
                      histtype='stepfilled', normed=True, **kwargs)
-            if self.tag is not None:
+            # if self.tag is not None:
                 # plt.suptitle('Histogram of ('+self.tag+')')
-                plt.title(str(self), fontsize=12)
-            else:
+                # plt.title(str(self), fontsize=12)
+            # else:
                 # plt.suptitle('Histogram of')
-                plt.title(str(self), fontsize=12)
+                # plt.title(str(self), fontsize=12)
             plt.ylim(0, 1.1*h[0].max())
         else:
             plt.plot(xp, p.evaluate(xp), **kwargs)
-            if self.tag is not None:
+            # if self.tag is not None:
                 # plt.suptitle('KDE of ('+self.tag+')')
-                plt.title(str(self), fontsize=12)
-            else:
+                # plt.title(str(self), fontsize=12)
+            # else:
                 # plt.suptitle('KDE of')
-                plt.title(str(self), fontsize=12)
+                # plt.title(str(self), fontsize=12)
 
         plt.xlim(low - (high - low)*0.1, high + (high - low)*0.1)
 
@@ -496,6 +507,8 @@ class UncertainVariable(UncertainFunction):
     +---------------------------+---------------------------------------------------------------+
     | Tri(a, b, c)              | http://en.wikipedia.org/wiki/Triangular_distribution          |
     +---------------------------+---------------------------------------------------------------+
+    | PERT(a, b, c)             | (This is based on the Beta distribution)                      |
+    +---------------------------+---------------------------------------------------------------+
     | T(df)                     | http://en.wikipedia.org/wiki/Student's_t-distribution         |
     +---------------------------+---------------------------------------------------------------+
     | Weib(lamda, k)            | http://en.wikipedia.org/wiki/Weibull_distribution             |
@@ -576,7 +589,8 @@ class UncertainVariable(UncertainFunction):
     
     See Also
     --------
-    N, U, Exp, Gamma, Beta, LogN, X2, F, Tri, T, Weib, Bern, B, G, H, Pois
+    N, U, Exp, Gamma, Beta, LogN, X2, F, Tri, PERT, T, Weib, Bern, B, G, H, 
+    Pois
         
     """
     
@@ -611,12 +625,12 @@ class UncertainVariable(UncertainFunction):
             h = plt.hist(vals, bins=np.round(np.sqrt(len(vals))), 
                      histtype='stepfilled', normed=True, **kwargs)
 
-            if self.tag is not None:
+            # if self.tag is not None:
                 # plt.suptitle('Histogram of (' + self.tag + ')')
-                plt.title(str(self), fontsize=12)
-            else:
+                # plt.title(str(self), fontsize=12)
+            # else:
                 # plt.suptitle('Histogram of')
-                plt.title(str(self), fontsize=12)
+                # plt.title(str(self), fontsize=12)
 
             plt.ylim(0, 1.1*h[0].max())
         else:
@@ -629,23 +643,23 @@ class UncertainVariable(UncertainFunction):
                 vals = range(low, high + 1)
                 plt.plot(vals, self.rv.pmf(vals), 'o', **kwargs)
 
-                if self.tag is not None:
+                # if self.tag is not None:
                     # plt.suptitle('PMF of (' + self.tag + ')')
-                    plt.title(str(self), fontsize=12)
-                else:
+                    # plt.title(str(self), fontsize=12)
+                # else:
                     # plt.suptitle('PMF of')
-                    plt.title(str(self), fontsize=12)
+                    # plt.title(str(self), fontsize=12)
 
             else:
                 vals = np.linspace(low, high, 500)
                 plt.plot(vals, self.rv.pdf(vals), **kwargs)
 
-                if self.tag is not None:
+                # if self.tag is not None:
                     # plt.suptitle('PDF of ('+self.tag+')')
-                    plt.title(str(self), fontsize=12)
-                else:
+                    # plt.title(str(self), fontsize=12)
+                # else:
                     # plt.suptitle('PDF of')
-                    plt.title(str(self), fontsize=12)
+                    # plt.title(str(self), fontsize=12)
 
         plt.xlim(low - (high - low)*0.1, high + (high - low)*0.1)
 
@@ -655,7 +669,9 @@ uv = UncertainVariable # a nicer form for the user
 
 # DON'T MOVE THIS IMPORT!!! The prior definitions must be in place before
 # importing the correlation-related functions
-from correlate import *
+from .correlate import *
+import umath
+import stats
 
 ###############################################################################
 # Define some convenience constructors for common statistical distributions.
@@ -663,39 +679,127 @@ from correlate import *
 # scipy.stats.distributions.
 ###############################################################################
 
-def N(mu, sigma, tag=None):
+###############################################################################
+# CONTINUOUS DISTRIBUTIONS
+###############################################################################
+
+def Beta(alpha, beta, low=0, high=1, tag=None):
     """
-    A Normal (or Gaussian) random variate
+    A Beta random variate
     
     Parameters
     ----------
-    mu : scalar
-        The mean value of the distribution
-    sigma : scalar
-        The standard deviation (must be positive and non-zero)
+    alpha : scalar
+        The first shape parameter
+    beta : scalar
+        The second shape parameter
+    
+    Optional
+    --------
+    low : scalar
+        Lower bound of the distribution support (default=0)
+    high : scalar
+        Upper bound of the distribution support (default=1)
     """
-    assert sigma>0, 'Normal "sigma" must be greater than zero'
-    return uv(ss.norm(loc=mu, scale=sigma), tag=tag)
+    assert alpha>0 and beta>0, 'Beta "alpha" and "beta" parameters must be greater than zero'
+    assert low<high, 'Beta "low" must be less than "high"'
+    return uv(ss.beta(alpha, beta, loc=low, scale=high-low), tag=tag)
 
 ###############################################################################
 
-def U(a, b, tag=None):
+def Bradford(q, low=0, high=1, tag=None):
     """
-    A Uniform random variate
+    A Bradford random variate
     
     Parameters
     ----------
-    a : scalar
-        Lower bound of the distribution support.
-    b : scalar
-        Upper bound of the distribution support.
+    q : scalar
+        The shape parameter
+    low : scalar
+        The lower bound of the distribution (default=0)
+    high : scalar
+        The upper bound of the distribution (default=1)
     """
-    assert a<b, 'Uniform "a" must be less than "b"'
-    return uv(ss.uniform(loc=a, scale=b-a), tag=tag)
+    assert q>0, 'Bradford "q" parameter must be greater than zero'
+    assert low<high, 'Bradford "low" parameter must be less than "high"'
+    return uv(ss.bradford(q, loc=low, scale=high-low), tag=tag)
 
 ###############################################################################
 
-def Exp(lamda, tag=None):
+def Burr(c, k, tag=None):
+    """
+    A Burr random variate
+    
+    Parameters
+    ----------
+    c : scalar
+        The first shape parameter
+    k : scalar
+        The second shape parameter
+    
+    """
+    assert c>0 and k>0, 'Burr "c" and "k" parameters must be greater than zero'
+    return uv(ss.burr(c, k), tag=tag)
+
+###############################################################################
+
+def ChiSquared(k, tag=None):
+    """
+    A Chi-Squared random variate
+    
+    Parameters
+    ----------
+    k : int
+        The degrees of freedom of the distribution (must be greater than one)
+    """
+    assert int(k)==k and k>=1, 'Chi-Squared "k" must be an integer greater than 0'
+    return uv(ss.chi2(k), tag=tag)
+
+Chi2 = ChiSquared  # for more concise use
+
+###############################################################################
+
+def Erf(h, tag=None):
+    """
+    An Error Function random variate. 
+    
+    This distribution is derived from a normal distribution by setting 
+    m = 0 and s = 1/(h*sqrt(2)), and thus is used in similar situations 
+    as the normal distribution.
+    
+    Parameters
+    ----------
+    h : scalar
+        The scale parameter.
+    """
+    assert h>0, 'Erf "h" must be greater than zero'
+    return Normal(0, 1/(h*2**0.5), tag)
+
+###############################################################################
+
+def Erlang(k, lamda, tag=None):
+    """
+    An Erlang random variate.
+    
+    This distribution is the same as a Gamma(k, theta) distribution, but 
+    with the restriction that k must be a positive integer. This
+    is provided for greater compatibility with other simulation tools, but
+    provides no advantage over the Gamma distribution in its applications.
+    
+    Parameters
+    ----------
+    k : int
+        The shape parameter (must be a positive integer)
+    lamda : scalar
+        The scale parameter (must be greater than zero)
+    """
+    assert int(k)==k and k>0, 'Erlang "k" must be a positive integer'
+    assert lamda>0, 'Erlang "lamda" must be greater than zero'
+    return Gamma(k, lamda, tag)
+
+###############################################################################
+
+def Exponential(lamda, tag=None):
     """
     An Exponential random variate
     
@@ -705,7 +809,66 @@ def Exp(lamda, tag=None):
         The inverse scale (as shown on Wikipedia). (FYI: mu = 1/lamda.)
     """
     assert lamda>0, 'Exponential "lamda" must be greater than zero'
-    return uv(ss.expon(scale=1./lamda), tag=tag)
+    return uv(ss.expon(scale=1.0/lamda), tag=tag)
+
+Exp = Exponential  # for more concise use
+
+###############################################################################
+
+def ExtValueMax(mu, sigma, tag=None):
+    """
+    An Extreme Value Maximum random variate.
+    
+    Parameters
+    ----------
+    mu : scalar
+        The location parameter
+    sigma : scalar
+        The scale parameter (must be greater than zero)
+    """
+    assert sigma>0, 'ExtremeValueMax "sigma" must be greater than zero'
+    p = U(0, 1)._mcpts[:]
+    return UncertainFunction(mu - sigma*np.log(-np.log(p)), tag=tag)
+
+EVMax = ExtValueMax  # for more concise use
+
+###############################################################################
+
+def ExtValueMin(mu, sigma, tag=None):
+    """
+    An Extreme Value Minimum random variate.
+    
+    Parameters
+    ----------
+    mu : scalar
+        The location parameter
+    sigma : scalar
+        The scale parameter (must be greater than zero)
+    """
+    assert sigma>0, 'ExtremeValueMin "sigma" must be greater than zero'
+    p = U(0, 1)._mcpts[:]
+    return UncertainFunction(mu + sigma*np.log(-np.log(1-p)), tag=tag)
+
+EVMin = ExtValueMin  # for more concise use
+
+###############################################################################
+
+def Fisher(d1, d2, tag=None):
+    """
+    An F (fisher) random variate
+    
+    Parameters
+    ----------
+    d1 : int
+        Numerator degrees of freedom
+    d2 : int
+        Denominator degrees of freedom
+    """
+    assert int(d1)==d1 and d1>=1, 'Fisher (F) "d1" must be an integer greater than 0'
+    assert int(d2)==d2 and d2>=1, 'Fisher (F) "d2" must be an integer greater than 0'
+    return uv(ss.f(d1, d2), tag=tag)
+
+F = Fisher  # for more concise use
 
 ###############################################################################
 
@@ -725,30 +888,7 @@ def Gamma(k, theta, tag=None):
 
 ###############################################################################
 
-def Beta(alpha, beta, a=0, b=1, tag=None):
-    """
-    A Beta random variate
-    
-    Parameters
-    ----------
-    alpha : scalar
-        The first shape parameter
-    beta : scalar
-        The second shape parameter
-    
-    Optional
-    --------
-    a : scalar
-        Lower bound of the distribution support (default=0)
-    b : scalar
-        Upper bound of the distribution support (default=1)
-    """
-    assert alpha>0 and beta>0, 'Beta "alpha" and "beta" parameters must be greater than zero'
-    return uv(ss.beta(alpha, beta, loc=a, scale=b-a), tag=tag)
-
-###############################################################################
-
-def LogN(mu, sigma, tag=None):
+def LogNormal(mu, sigma, tag=None):
     """
     A Log-Normal random variate
     
@@ -762,58 +902,57 @@ def LogN(mu, sigma, tag=None):
     assert sigma>0, 'Log-Normal "sigma" must be positive'
     return uv(ss.lognorm(sigma, loc=mu), tag=tag)
 
-###############################################################################
-
-def Chi2(k, tag=None):
-    """
-    A Chi-Squared random variate
-    
-    Parameters
-    ----------
-    k : int
-        The degrees of freedom of the distribution (must be greater than one)
-    """
-    assert isinstance(k, int) and k>=1, 'Chi-Squared "k" must be an integer greater than 0'
-    return uv(ss.chi2(k), tag=tag)
+LogN = LogNormal  # for more concise use
 
 ###############################################################################
 
-def F(d1, d2, tag=None):
+def Normal(mu, sigma, tag=None):
     """
-    An F (fisher) random variate
+    A Normal (or Gaussian) random variate
     
     Parameters
     ----------
-    d1 : int
-        Numerator degrees of freedom
-    d2 : int
-        Denominator degrees of freedom
+    mu : scalar
+        The mean value of the distribution
+    sigma : scalar
+        The standard deviation (must be positive and non-zero)
     """
-    assert isinstance(d1, int) and d1>=1, 'Fisher (F) "d1" must be an integer greater than 0'
-    assert isinstance(d2, int) and d2>=1, 'Fisher (F) "d2" must be an integer greater than 0'
-    return uv(ss.f(d1, d2), tag=tag)
+    assert sigma>0, 'Normal "sigma" must be greater than zero'
+    return uv(ss.norm(loc=mu, scale=sigma), tag=tag)
+
+N = Normal  # for more concise use
 
 ###############################################################################
 
-def Tri(a, b, c, tag=None):
+def PERT(low, peak, high, tag=None):
     """
-    A triangular random variate
+    A PERT random variate
     
     Parameters
     ----------
-    a : scalar
+    low : scalar
         Lower bound of the distribution support
-    b : scalar
+    peak : scalar
+        The location of the distribution's peak (low <= peak <= high)
+    high : scalar
         Upper bound of the distribution support
-    c : scalar
-        The location of the triangle's peak (a <= c <= b)
     """
-    assert a<=c<=b, 'Triangular "c" must lie between "a" and "b"'
-    return uv(ss.triang((1.0*c-a)/(b-a), loc=a, scale=b-a), tag=tag)
+    a = low
+    b = peak
+    c = high
+    assert a<=b<=c, 'PERT "peak" must be greater than "low" and less than "high"'
+    mu = (a + 4.0*b + c)/6
+    if mu==b:
+        a1 = a2 = 3.0
+    else:
+        a1 = ((mu - a)*(2*b - a - c))/((b - mu)*(c - a))
+        a2 = a1*(c - mu)/(mu - a)
+        
+    return Beta(a1, a2, a, c, tag)
 
 ###############################################################################
 
-def T(v, tag=None):
+def StudentT(v, tag=None):
     """
     A Student-T random variate
     
@@ -822,12 +961,53 @@ def T(v, tag=None):
     v : int
         The degrees of freedom of the distribution (must be greater than one)
     """
-    assert isinstance(v, int) and v>=1, 'Student-T "v" must be an integer greater than 0'
+    assert int(v)==v and v>=1, 'Student-T "v" must be an integer greater than 0'
     return uv(ss.t(v), tag=tag)
 
+T = StudentT  # for more concise use
+
 ###############################################################################
 
-def Weib(lamda, k, tag=None):
+def Triangular(low, peak, high, tag=None):
+    """
+    A triangular random variate
+    
+    Parameters
+    ----------
+    low : scalar
+        Lower bound of the distribution support
+    peak : scalar
+        The location of the triangle's peak (low <= peak <= high)
+    high : scalar
+        Upper bound of the distribution support
+    """
+    assert low<=peak<=high, 'Triangular "peak" must lie between "low" and "high"'
+    return uv(ss.triang((1.0*peak - low)/(high - low), loc=low, 
+        scale=(high - low)), tag=tag)
+
+Tri = Triangular  # for more concise use
+
+###############################################################################
+
+def Uniform(low, high, tag=None):
+    """
+    A Uniform random variate
+    
+    Parameters
+    ----------
+    low : scalar
+        Lower bound of the distribution support.
+    high : scalar
+        Upper bound of the distribution support.
+    """
+    assert low<high, 'Uniform "low" must be less than "high"'
+    return uv(ss.uniform(loc=low, scale=high-low), tag=tag)
+
+U = Uniform  # for more concise use
+
+###############################################################################
+
+def Weibull(lamda, k, tag=None):
     """
     A Weibull random variate
     
@@ -841,9 +1021,13 @@ def Weib(lamda, k, tag=None):
     assert lamda>0 and k>0, 'Weibull "lamda" and "k" parameters must be greater than zero'
     return uv(ss.exponweib(lamda, k), tag=tag)
 
+Weib = Weibull  # for more concise use
+
+###############################################################################
+# DISCRETE DISTRIBUTIONS
 ###############################################################################
 
-def Bern(p, tag=None):
+def Bernoulli(p, tag=None):
     """
     A Bernoulli random variate
     
@@ -855,9 +1039,11 @@ def Bern(p, tag=None):
     assert 0<p<1, 'Bernoulli probability "p" must be between zero and one, non-inclusive'
     return uv(ss.bernoulli(p), tag=tag)
 
+Bern = Bernoulli  # for more concise use
+
 ###############################################################################
 
-def B(n, p, tag=None):
+def Binomial(n, p, tag=None):
     """
     A Binomial random variate
     
@@ -872,9 +1058,11 @@ def B(n, p, tag=None):
     assert 0<p<1, 'Binomial probability "p" must be between zero and one, non-inclusive'
     return uv(ss.binom(n, p), tag=tag)
 
+B = Binomial  # for more concise use
+
 ###############################################################################
 
-def G(p, tag=None):
+def Geometric(p, tag=None):
     """
     A Geometric random variate
     
@@ -886,9 +1074,11 @@ def G(p, tag=None):
     assert 0<p<1, 'Geometric probability "p" must be between zero and one, non-inclusive'
     return uv(ss.geom(p), tag=tag)
 
+G = Geometric  # for more concise use
+
 ###############################################################################
 
-def H(N, n, K, tag=None):
+def Hypergeometric(N, n, K, tag=None):
     """
     A Hypergeometric random variate
     
@@ -927,9 +1117,11 @@ def H(N, n, K, tag=None):
     assert int(K)==K and 0<K<=N, 'Hypergeometric chosen population size "K" must be an integer greater than zero and no more than the total population size.'
     return uv(ss.hypergeom(N, n, K), tag=tag)
 
+H = Hypergeometric  # for more concise use
+
 ###############################################################################
 
-def Pois(lamda, tag=None):
+def Poisson(lamda, tag=None):
     """
     A Poisson random variate
     
@@ -941,6 +1133,10 @@ def Pois(lamda, tag=None):
     assert lamda>0, 'Poisson "lamda" must be greater than zero.'
     return uv(ss.poisson(lamda), tag=tag)
 
+Pois = Poisson  # for more concise use
+
+###############################################################################
+# STATISTICAL FUNCTIONS
 ###############################################################################
 
 def covariance_matrix(nums_with_uncert):
@@ -988,6 +1184,8 @@ def covariance_matrix(nums_with_uncert):
 
     return cov_matrix
 
+###############################################################################
+
 def correlation_matrix(nums_with_uncert):
     """
     Calculate the correlation matrix of uncertain variables, oriented by the
